/*  queue.c
 *
 *      Implementation of a FIFO queue abstract data type.
 *
 *          by: Steven Skiena
 *              begun: March 27, 2002
 *              */


/*
 * Copyright 2003 by Steven S. Skiena; all rights reserved. 
 *
 * Permission is granted for use in non-commerical applications
 * provided this copyright notice remains intact and unchanged.
 *
 * This program appears in my book:
 *
 * "Programming Challenges: The Programming Contest Training Manual"
 * by Steven Skiena and Miguel Revilla, Springer-Verlag, New York 2003.
 *
 * See our website www.programming-challenges.com for additional information.
 *
 * This book can be ordered from Amazon.com at
 *
 * http://www.amazon.com/exec/obidos/ASIN/0387001638/thealgorithmrepo/
 *
 * */

/*  queue.h
 *
 *      Header file for queue implementation
 *
 *          by: Steven Skiena
 *          */

/*
 * Copyright 2003 by Steven S. Skiena; all rights reserved. 
 *
 * Permission is granted for use in non-commerical applications
 * provided this copyright notice remains intact and unchanged.
 *
 * This program appears in my book:
 *
 * "Programming Challenges: The Programming Contest Training Manual"
 * by Steven Skiena and Miguel Revilla, Springer-Verlag, New York 2003.
 *
 * See our website www.programming-challenges.com for additional information.
 *
 * This book can be ordered from Amazon.com at
 *
 * http://www.amazon.com/exec/obidos/ASIN/0387001638/thealgorithmrepo/
 *
 * */

#include <stdio.h>
#define QUEUESIZE       1000

typedef struct {
        int q[QUEUESIZE+1];     /* body of queue */
        int first;                      /* position of first element */
        int last;                       /* position of last element */
        int count;                      /* number of queue elements */
} queue;

#define TRUE    1
#define FALSE   0

typedef int bool;

void init_queue(queue *q)
{
        q->first = 0;
        q->last = QUEUESIZE-1;
        q->count = 0;
}

void enqueue(queue *q, int x)
{
        if (q->count >= QUEUESIZE)
        printf("Warning: queue overflow enqueue x=%d\n",x);
        else {
                q->last = (q->last+1) % QUEUESIZE;
                q->q[ q->last ] = x;    
                q->count = q->count + 1;
        }
}

int dequeue(queue *q)
{
        int x;

        if (q->count <= 0) printf("Warning: empty queue dequeue.\n");
        else {
                x = q->q[ q->first ];
                q->first = (q->first+1) % QUEUESIZE;
                q->count = q->count - 1;
        }

        return(x);
}

int empty(queue *q)
{
        if (q->count <= 0) return (TRUE);
        else return (FALSE);
}

void print_queue(queue *q)
{
        int i,j;

        i=q->first; 
        
        while (i != q->last) {
                printf("%d ",q->q[i]);
                i = (i+1) % QUEUESIZE;
        }

        printf("%2d ",q->q[i]);
        printf("\n");
}

int main(){
    queue q;
    init_queue(&q);
    printf("enqueuing 1\n");
    enqueue(&q, 1);
    printf("enqueuing 2\n");
    enqueue(&q, 2);
    printf("enqueuing 3\n");
    enqueue(&q, 3);
    
    printf("queue now looks like \n");
    print_queue(&q);

    printf("dequeuing returned %d \n", dequeue(&q));
    
    printf("queue now looks like \n");
    print_queue(&q);
    
    printf("dequeuing returned %d \n", dequeue(&q));
    
    printf("queue now looks like \n");
    print_queue(&q);

    return 0;    
}
